#include "DXE_CLONE.CH"
//////////////////////////////////////////////////////////////////////
//
//  IMGVIEW.PRG
//
//  Copyright:
//      Alaska Software, (c) 1997-2006. Alle Rechte vorbehalten.
//
//  Inhalt:
//      Das Programm enthlt drei Klassen, die sich aus verschiedenen
//      Xbase Parts zusammensetzen:
//
//      o ImageView    -  Anzeige von Bitmaps oder Metadateien in
//                        einem XbpStatic Objekt
//
//      o DirSelector  -  Auswahl von Laufwerk und Verzeichnis
//                        (Kombination von XbpCombobox und XbpTreeView)
//
//      o FileSelector -  Auswahl von Dateien anhand einer Liste
//                        von Dateierweiterungen
//                        (Kombination von XbpCombobox und XbpListbox)
//
//      Objekte dieser Klassen werden in einem Dialogfenster kombiniert,
//      so da Bitmaps in beliebigen Verzeichnissen gesucht und angezeigt
//      werden knnen.
//
//////////////////////////////////////////////////////////////////////

#include "Appevent.ch"
#include "Directry.ch"
#include "Imgview.ch"
#include "Common.ch"
#include "Font.ch"
#include "Gra.ch"
#include "Xbp.ch"


/*
 * Im Programm werden benutzerdefinierte Ereignisse verwendet
 */
#define  xbeDS_DirChanged           xbeP_User + 100
#define  xbeFS_FileMarked           xbeP_User + 101
#define  xbeFS_FileSelected         xbeP_User + 102

/*
 * Die Direktive berechnet eine zentrierte Position
 */
#xtrans  CenterPos( <aSize>, <aRefSize> ) => ;
         { Int( (<aRefSize>\[1] - <aSize>\[1]) / 2 ) ;
         , Int( (<aRefSize>\[2] - <aSize>\[2]) / 2 ) }



/*
 * Appsys bleibt hier leer
 */
PROCEDURE AppSys
RETURN



/*
 * Prozedur zum Abbruch des Programms
 */
PROCEDURE AppQuit
   QUIT
RETURN




/*
 * Die Main Prozedur erzeugt den Image View Dialog
 */
PROCEDURE Main
   LOCAL nEvent, mp1, mp2, oXbp
   LOCAL oDlg, drawingArea, aPos, aSize
   LOCAL oDirs, oFiles, oView
   LOCAL oRadio1, oRadio2



   SET ALTER TO _LOGFILE.TXT
   SET ALTER ON

   aSize := {590,360}
   aPos  := CenterPos( aSize, AppDesktop():currentSize() )

   oDlg := XbpDialog():new( AppDesktop(), , aPos, aSize, , .F. )
   oDlg:taskList := .T.
   oDlg:maxButton := .F.
   oDlg:border   := XBPDLG_DLGBORDER
   oDlg:title    := "Image viewer"
   oDlg:close    := {|| AppQuit() }
   oDlg:create()
   SetAppWindow( oDlg )

   drawingArea := oDlg:drawingArea
   drawingArea:setFontCompoundName( FONT_HELV_SMALL )

   oDirs  := DirSelector():new ( drawingArea, , {12 ,12}, {168,312} )
   oDirs:tabStop := .T.
   oDirs:startDir := Upper( getenv("XPPRESOURCE") )
   oDirs:create()

   oFiles := FileSelector():new( drawingArea, , {192,12}, {168,312} )
   oFiles:tabStop := .T.
   oFiles:create()

   oFiles:setFilesTo( oDirs:curDir(), { "*.BMP", "*.EMF", "*.GIF", "*.JPG", "*.PNG", "*.BMP *.EMF *.GIF *.JPG *.PNG"  } )

   oView := ImageView():new( drawingArea, , {372,96}, {204,228} ):create()

   oRadio1 := XbpRadioButton():new( drawingArea, , {387,60}, {72,24} )
   oRadio1:caption   := "Skaliert"
   oRadio1:tabStop   := .T.
   oRadio1:selection := oView:autoScale
   oRadio1:create()
   oRadio1:selected  := {|| oRadio2:SetData(.F.), oRadio1:SetData(.T.), ;
                            oView:autoScale := .T., oView:display() }

   oRadio2 := XbpRadioButton():new( drawingArea, , {487,60}, {72,24} )
   oRadio2:caption   := "Normal"
   oRadio2:tabStop   := .T.
   oRadio2:selection := ! oView:autoScale
   oRadio2:create()
   oRadio2:selected  := {|| oRadio1:SetData(.F.), oRadio2:SetData(.T.), ;
                            oView:autoScale := .F., oView:display() }

   oXbp := XbpStatic():new( drawingArea, , {372,48}, {204,2} )
   oXbp:type := XBPSTATIC_TYPE_RAISEDLINE
   oXbp:create()

   oXbp := XbpPushButton():new( drawingArea, , {377,12}, {72,24} )
   oXbp:caption  := "Vollbild"
   oXbp:tabStop  := .T.
   oXbp:create()
   oXbp:activate := {|| FullView( oView:file ) }

   oXbp := XbpPushButton():new( drawingArea, , {497,12}, {72,24} )
   oXbp:caption  := "Ende"
   oXbp:tabStop  := .T.
   oXbp:create()
   oXbp:activate := {|| AppQuit() }

   oDlg:show()
   SetAppFocus( oDirs:dirTree )

   DO WHILE .T.
      nEvent := AppEvent( @mp1, @mp2, @oXbp )

      IF nEvent == xbeDS_DirChanged
         oFiles:setFilesTo( mp1 )

      ELSEIF nEvent == xbeFS_FileMarked
         oView:load( mp1 )
         oView:display()

      ELSE
          oXbp:handleEvent( nEvent, mp1, mp2 )
      ENDIF
   ENDDO

   SET ALTER OFF
   SET ALTER TO

RETURN



/*
 * Die Prozedur zeigt eine Bilddatei in einem eigenen Fenster an
 */
PROCEDURE FullView( cFile )
   LOCAL oDlg, oImage, oPS, aSize, aPos
   LOCAL lBGClr := XBPSYSCLR_TRANSPARENT

  /*
   * Es werden nur Bitmap und Metadateien untersttzt
   */
   IF cFile <> NIL            .AND. ;
     ( ".BMP" $ Upper( cFile ) .OR. ;
       ".EMF" $ Upper( cFile ) .OR. ;
       ".GIF" $ Upper( cFile ) .OR. ;
       ".JPG" $ Upper( cFile ) .OR. ;
       ".PNG" $ Upper( cFile ) .OR. ;
       ".MET" $ Upper( cFile )      )

     /*
      * Dialog versteckt erzeugen
      */
      oDlg := XbpDialog():new( AppDesktop(),,,{100,100} )
      oDlg:taskList   := .F.
      oDlg:visible    := .F.
      oDlg:title      := cFile
      oDlg:sizeRedraw := .T.
      oDlg:close      := {|mp1,mp2,obj| obj:destroy() }
      oDlg:create()

     /*
      * Presentation Space erzeugen, der mit dem Device-Kontext der
      * drawingArea verknpft wird
      */
      oPS := XbpPresSpace():new():create( oDlg:drawingArea:winDevice() )

      IF ".BMP" $ Upper( cFile ) .OR. ;
         ".GIF" $ Upper( cFile ) .OR. ;
         ".JPG" $ Upper( cFile ) .OR. ;
         ".PNG" $ Upper( cFile )

        /*
         * Datei enthlt eine Bitmap. Die Gre des Fensters wird
         * begrenzt auf 16x16 Pixel und die Bildschirmauflsung
         */
         oImage   := XbpBitmap():new():create( oPS )
         oImage:loadFile( cFile )

         IF oImage:transparentClr <> GRA_CLR_INVALID
            lBGClr := XBPSYSCLR_DIALOGBACKGROUND
         ENDIF

         aSize    := { oImage:xSize, oImage:ySize }
         aSize[1] := Max( 16, Min( aSize[1], AppDeskTop():currentSize()[1] ) )
         aSize[2] := Max( 16, Min( aSize[2], AppDeskTop():currentSize()[2] ) )
         aSize    := oDlg:calcFrameRect( {0,0, aSize[1], aSize[2]} )

         oDlg:setSize( {aSize[3], aSize[4]} )

         /*
          * Das Fenster mu auf xbeP_Paint reagieren und
          * die Bitmap neu anzeigen
          */
         oDlg:drawingarea:paint := {|x,y,obj| x:=obj:currentSize(), ;
                                     oImage:draw( oPS, {0, 0, x[1], x[2]}, ;
                                     {0, 0, oImage:xSize, oImage:ySize},,;
                                     GRA_BLT_BBO_IGNORE), Sleep(0.1) }
      ELSE
        /*
         * Metadatei anzeigen. Sie haben keine feste Grendefinition
         */
         oImage := XbpMetafile():new():create()
         oImage:load( cFile )
         aSize := { 600, 400 }
         oDlg:setSize( aSize )
         oDlg:drawingarea:paint := {|x,y,obj| x:=obj:currentSize(), ;
                                              oImage:draw( oPS, {0, 0, x[1], x[2]}),;
                                              Sleep(0.1) }
         lBGClr := XBPSYSCLR_DIALOGBACKGROUND
      ENDIF

     /*
      * Setzen der Hintergrundfarbe der drawingarea des Dialogs.
      * In den meisten Fllen wird hier die Transparenzfarbe
      * verwendet, um Flackern beim Zeichnen zu vermeiden. Fr
      * transparente Bilder und Meta-Dateien wird hier allerdings
      * die Farbe Grau gesetzt, siehe oben. Dies verhindert, da
      * Teile des Desktops in transparenten Bereichen der Grafik
      * sichtbar sind. Alternativ knnte fr Bitmap-Bilder die
      * Transparenz explizit ausgeschaltet werden.
      */
      oDlg:drawingArea:SetColorBG( lBGClr )

     /*
      * Fenster zentriert auf dem Desktop anzeigen
      */
      aPos:= CenterPos( oDlg:currentSize(), AppDesktop():currentSize() )
      oDlg:setPos( aPos )
      oDlg:show()
      SetAppFocus( oDlg )
   ENDIF
RETURN



/*
 * Klasse zum Anzeigen von Bitmaps oder Metadateien
 */
CLASS ImageView FROM XbpStatic
   PROTECTED:
   VAR oFrame
   VAR oCanvas
   VAR oPS
   VAR oBitmap
   VAR oMetafile
   VAR nMode

   EXPORTED:
   VAR autoScale
   VAR file
   METHOD init, create, load, display
ENDCLASS



/*
 * Objekt initialisieren. Self ist Parent fr zwei XbpStatic Objekte
 */
METHOD ImageView:init( oParent, oOwner, aPos, aSize, aPresParam, lVisible )
   ::xbpStatic:init( oParent, oOwner, aPos, aSize, aPresParam, lVisible )
   ::xbpStatic:type := XBPSTATIC_TYPE_RAISEDRECT

   ::oFrame         := XbpStatic():new( self )
   ::oFrame:type    := XBPSTATIC_TYPE_RECESSEDRECT

   ::oCanvas        := XbpStatic():new( self )
   ::autoScale      := .T.
   ::nMode          := 0
   ::oPS            := XbpPresSpace():new()
RETURN self



/*
 * Systemresourcen anfordern. Self ist der uere Rahmen, ::oFrame ist der
 * innere Rahmen und in ::oCanvas wird das Bild gezeichnet
 */
METHOD ImageView:create( oParent, oOwner, aPos, aSize, aPresParam, lVisible )
   LOCAL aAttr[ GRA_AA_COUNT ]

   ::xbpStatic:create( oParent, oOwner, aPos, aSize, aPresParam, lVisible )
   aSize    := ::currentSize()
   aSize[1] -= 8
   aSize[2] -= 8

   ::oFrame:create(self,, {4,4}, aSize )

   aSize[1] -= 2
   aSize[2] -= 2

   ::oCanvas:create(::oFrame,, {1,1}, aSize )

  /*
   * Presentation Space mit dem Device Kontext von XbpStatic verbinden
   */
   ::oPS:create( ::oCanvas:winDevice() )
   aAttr[ GRA_AA_COLOR ] := GRA_CLR_BACKGROUND
   ::oPS:setAttrArea( aAttr )
   ::oCanvas:paint := {| aClip | ::display( aClip ) }

RETURN self



/*
 * Bilddatei laden
 */
METHOD ImageView:load( cFile )
   LOCAL lSuccess := .F.

   IF Valtype( cFile ) <> "C" .OR. ! File( cFile )
      RETURN lSuccess
   ENDIF

   ::nMode := 0
   ::file  := ""

   IF ".BMP" $ Upper( cFile ) .OR. ;
      ".GIF" $ Upper( cFile ) .OR. ;
      ".JPG" $ Upper( cFile ) .OR. ;
      ".PNG" $ Upper( cFile )

      IF ::oBitmap == NIL
         ::oBitmap := XbpBitmap():New():create( ::oPS )
      ENDIF

      IF ( lSuccess := ::oBitmap:loadFile( cFile ) )
         ::nMode := 1
         ::file := cFile
      ENDIF

   ELSEIF ".EMF" $ Upper( cFile ) .OR. ".MET" $ Upper( cFile )
      IF ::oMetafile == NIL
         ::oMetafile := XbpMetafile():New():create()
      ENDIF

      IF ( lSuccess := ::oMetafile:load( cFile ) )
         ::nMode := 2
         ::file := cFile
      ENDIF
   ENDIF

RETURN lSuccess



/*
 * Bilddatei anzeigen
 */
METHOD ImageView:display( aClip )
   LOCAL lSuccess := .F.
   LOCAL aSize    := ::oCanvas:currentSize()
   LOCAL aTarget, aSource, nAspect

   /*
    * Clip Bereich vorbereiten
    */
   DEFAULT aClip TO { 1, 1, aSize[1]-1, aSize[2]-1 }
   GraPathBegin( ::oPS )
   GraBox( ::oPS, { aClip[1]-1, aClip[2]-1 }, { aClip[3]+1, aClip[4]+1 }, GRA_OUTLINE )
   GraPathEnd( ::oPS )
   GraPathClip( ::oPS, .T. )

   GraBox( ::oPS, {0,0}, aSize, GRA_FILL )

   DO CASE
   CASE ::nMode == 1
     /*
      * Eine Bitmap Datei ist geladen
      */
      aSource := {0,0,::oBitmap:xSize,::oBitmap:ySize}
      aTarget := {1,1,aSize[1]-2,aSize[2]-2}

      IF ::autoScale
        /*
         * Bitmap wird auf die Gre von ::oCanvas skaliert
         */
         nAspect    := aSource[3] / aSource[4]
         IF nAspect > 1
            aTarget[4] := aTarget[3] / nAspect
         ELSE
            aTarget[3] := aTarget[4] * nAspect
         ENDIF
      ELSE
         aTarget[3] := aSource[3]
         aTarget[4] := aSource[4]
      ENDIF


     /*
      * Bitmap in ::oCanvas horizontal oder vertikal zentrieren
      */
      IF aTarget[3] < aSize[1]-2
         nAspect := ( aSize[1]-2-aTarget[3] ) / 2
         aTarget[1] += nAspect
         aTarget[3] += nAspect
      ENDIF

      IF aTarget[4] < aSize[2]-2
         nAspect := ( aSize[2]-2-aTarget[4] ) / 2
         aTarget[2] += nAspect
         aTarget[4] += nAspect
      ENDIF

      ::oBitmap:draw( ::oPS, aTarget, aSource, , GRA_BLT_BBO_IGNORE )
      Sleep(1)

   CASE ::nMode == 2
     /*
      * Eine Metadatei ist geladen
      */
      IF ::autoScale
         ::oMetafile:draw( ::oPS, XBPMETA_DRAW_SCALE )
      ELSE
         ::oMetafile:draw( ::oPS, XBPMETA_DRAW_DEFAULT )
      ENDIF

   ENDCASE

   GraPathClip( ::oPS, .F. )

RETURN lSuccess




/*
 * Klasse zur auswahl von Laufwerken und Verzeichnissen
 */
CLASS DirSelector FROM XbpStatic
   PROTECTED:
   VAR aDrives
   VAR oItem

   EXPORTED:
   VAR startDir
   VAR dirTree
   VAR driveList

   METHOD init, create, fillTree, curDir
ENDCLASS



/*
 * Objekt initialisieren
 */
METHOD DirSelector:init( oParent, oOwner, aPos, aSize, aPresParam, lVisible )

   ::xbpStatic:init( oParent, oOwner, aPos, aSize, aPresParam, lVisible )
   ::xbpStatic:type    := XBPSTATIC_TYPE_RAISEDRECT

   ::dirTree           := XbpTreeView():new( self )
   ::dirTree:tabStop   := .T.
   ::dirTree:hasLines  := .T.
   ::dirTree:hasButtons:= .T.

   ::driveList         := XbpComboBox():new( self )
   ::driveList:tabStop := .T.
   ::driveList:type    := XBPCOMBO_DROPDOWNLIST

   ::startDir          := CurDrive() + ":\" + CurDir()
RETURN self



/*
 * Systemresourcen anfordern und Daten fr das aktuelle Verzeichnis
 * in die Treeview eintragen
 */
METHOD DirSelector:create( oParent, oOwner, aPos, aSize, aPresParam, lVisible )

   LOCAL nDrive

   ::xbpStatic:create( oParent, oOwner, aPos, aSize, aPresParam, lVisible )

   aSize := ::currentSize()

  /*
   * Die Treeview erzeugt ein benutzerdefiniertes Ereignis und
   * fgt das gewhlte Verzeichnis als ersten Message-Parameter hinzu
   */
   ::dirTree:create  (,, {4, 30}, {aSize[1]-8, aSize[2]-30-4 } )
   ::dirTree:itemSelected := {|oItem| ::fillTree( oItem ) }
   ::dirTree:itemMarked   := {|oItem| PostAppEvent( xbeDS_DirChanged, oItem:getData() ) }


   IF At(";", ::startDir) <> 0
      ::startDir := Left( ::startDir, At(";", ::startDir) -1 )
   ENDIF


   ::dirTree:rootItem:setData( ::startDir )

   ::driveList:create(,, {4, 30-4-24-96}, {aSize[1]-8, 24+96 } )

   ::aDrives := AvailDrives()
   AEval( ::aDrives, {|c| ::driveList:xbpListBox:addItem( c+":" ) } )

   IF At( ":", ::startDir ) == 2
      nDrive := AScan( ::aDrives, Left(::startDir,1) )
   ELSE
      nDrive := AScan( ::aDrives, CurDrive() )
   ENDIF

   IF nDrive == 0
      MsgBox( "Das fr das Bilderverzeichnis angegebene Laufwerk ist " +;
              "ungltig! Bitte berprfen Sie Ihre Installation und "  +;
              "stellen Sie sicher, da XPPRESOURCE einen gltigen "    +;
              "Pfad enthlt!" )

      ::startDir := CurDrive() + ":\" + CurDir()
      ::dirTree:rootItem:setData( ::startDir )
      nDrive     := AScan( ::aDrives, CurDrive() )
   ENDIF

   ::driveList:xbpListBox:setData( { nDrive } )
   ::driveList:xbpListBox:itemSelected := {|| ::fillTree() }

   ::fillTree( ::dirTree:rootItem )
RETURN self



/*
 * Treeview mit den Daten des gewhlten Verzeichnisses aufbauen.
 * Jedes XbpTreeViewItem enthlt eine Zeichenkette mit dem Verzeichnis,
 * das es anzeigt
 */
METHOD DirSelector:fillTree( oItem )
   LOCAL aItems    := ::dirTree:rootItem:getChildItems()
   LOCAL cDrive    := AllTrim( ::driveList:xbpSLE:getData() )
   LOCAL cDir, aDir, i, imax, nLen, oParent, oTemp, cTemp, oNewParent

   LOCAL cTXT1,cTXT2

   STATIC cLastDrive := ""

*   ondummy( "fillTree", oItem ,cDrive)

   IF oItem <> NIL .AND. ::isVisible() .AND. Len( Directory( oItem:getData() + "\*.", "D" ) ) == 2

      ondummy( "fillTree1"  ,;
               oItem        ,;
               ::isVisible(),;
               oItem:getData()+"\*.",;
               Directory( oItem:getData() + "\*.", "D" ) )

      RETURN self
   ENDIF

   ::dirTree:LockUpdate( .T. )

   AEval( aItems, {|obj| ::dirTree:rootItem:delItem(obj) } )
   cDir    := cDrive

   //
   // NEED for DXE when change Drive ???
   //
   IF cLastDrive <> cDrive
      IF oItem == NIL

         //
         //
         //
         ::dirTree:rootItem:ClearAll()
      ENDIF
      cLastDrive := cDrive
   ENDIF


  /*
   * Die Treeview beginnt mit dem Laufwerksbuchstaben
   */
   oParent := ::dirTree:rootItem:addItem( cDrive, ICON_DRIVE, ICON_DRIVE, ICON_DRIVE,, cDrive )

*   ondummy( "oItem", oItem ,"Parent", oParent:caption)

   IF oItem == NIL
      oItem := oParent
   ENDIF

   ::oItem := oItem

   cTXT1 := oItem:caption
   cTXT2 := oItem:getData()

   ondummy( "cDir", cDir , cTXT1 , cTXT2 )

IF cTXT1 == cTXT2
   ondummy( "cTXT1 == cTXT2", cTXT1 , cTXT2 , PROCNAME(1),PROCLINE(1) )
ENDIF

//ELSE
  /*
   * Fr jedes Unterverzeichnis wird ein XbpTreeviewItem erzeugt
   */
   DO WHILE .T.

      cTXT1 := Upper(cDir)
      cTXT2 := oItem:getData()

      IF NIL = cTXT2
*         ondummy("Error fillTree()",cTXT1,cDrive, oItem, oItem:caption, oItem:getData() )
         tone(1234)
         EXIT

      ENDIF

      cTXT2 := Upper(oItem:getData())

*      ondummy( cTXT1, "<>" ,cTXT2 )

      IF cTXT1 <> cTXT2
      ELSE
         EXIT
      ENDIF

      cTemp := oItem:getData()
*      ondummy( "cTemp1=",cTemp)

      cTemp := SubStr( oItem:getData(), Len(cDir)+2 )
*      ondummy( "cTemp2=",cTemp)


      nLen := At( "\", cTemp ) - 1
      IF nLen < 0
          nLen := len(cTemp)
      ENDIF
      cTemp := SubStr( cTemp, 1, nLen)
*      ondummy( "cTemp3=",cTemp )


      aDir := Directory( cDir + "\", "D" )
      ondummy( "cDir1", cDir ,"aDir",LEN(aDir) )

      imax := Len( aDir )
      FOR i:=1 TO imax
         IF "D" $ aDir[i,F_ATTR]        .AND. ;
            .NOT. aDir[i,F_NAME] == "." .AND. ;
            .NOT. aDir[i,F_NAME] == ".."
            oTemp := oParent:addItem( aDir[i,F_NAME]   , ;
                                      ICON_CLOSEDFOLDER, ;
                                      ICON_CLOSEDFOLDER, ;
                                      ICON_OPENFOLDER  , ;
                                      NIL              , ;
                                      cDir + "\" + aDir[i,F_NAME] )

            IF Upper( cDir + "\" + aDir[i,F_NAME] ) == Upper( SubStr( oItem:getData(), 1, Len(cDir + "\" + cTemp) ) )
               oNewParent := oTemp
            ENDIF
         ENDIF
      NEXT

      cDir += "\" + cTemp
      oParent:expand(.T.)
      IF oNewParent <> NIL
         oParent := oNewParent
      ENDIF
   ENDDO


  /*
   * XbpTreeViewItems fr die Dateien im untersten Verzeichnis erzeugen
   */
   aDir := Directory( cDir + "\", "D" )

   ondummy( "aDir", cDir + "\", LEN(aDir) )

   imax := Len( aDir )
   FOR i:=1 TO imax
      IF "D" $ aDir[i,F_ATTR]        .AND. ;
         .NOT. aDir[i,F_NAME] == "." .AND. ;
         .NOT. aDir[i,F_NAME] == ".."

         oParent:addItem( aDir[i,F_NAME]   , ;
                          ICON_CLOSEDFOLDER, ;
                          ICON_CLOSEDFOLDER, ;
                          ICON_OPENFOLDER  , ;
                          NIL              , ;
                          cDir + "\" + aDir[i,F_NAME] )
      ENDIF
   NEXT

   oParent:expand( .T. )
   ::dirTree:SetData( oParent )

   ::dirTree:LockUpdate( .F. )
   ::dirTree:InvalidateRect()

   PostAppEvent( xbeDS_DirChanged, oParent:getData() )

//ENDIF

RETURN self



/*
 * Aktuelles ausgewhltes Verzeichnis zurckgeben
 */
METHOD DirSelector:curDir
RETURN ::oItem:getData()



/*
 * Klasse zur Auswahlt von Dateien mit bestimmten Erweiterungen
 */
CLASS FileSelector FROM XbpStatic
   PROTECTED:
   VAR aFiles
   VAR aExtensions
   VAR oItem
   VAR cDir

   EXPORTED:
   VAR fileList
   VAR extList

   METHOD init, create, setFilesTo, fillFileList, fileMarked, fileSelected
ENDCLASS



/*
 * Objekt initialisieren
 */
METHOD FileSelector:init( oParent, oOwner, aPos, aSize, aPresParam, lVisible )

   ::xbpStatic:init( oParent, oOwner, aPos, aSize, aPresParam, lVisible )
   ::xbpStatic:type := XBPSTATIC_TYPE_RAISEDRECT

   ::fileList         := XbpListBox():new( self )
   ::fileList:tabStop := .T.
   ::extList          := XbpComboBox():new( self )
   ::extList:tabStop  := .T.
   ::extList:type     := XBPCOMBO_DROPDOWNLIST

RETURN self



/*
 * Systemresourcen anfordern
 */
METHOD FileSelector:create( oParent, oOwner, aPos, aSize, aPresParam, lVisible )
   ::xbpStatic:create( oParent, oOwner, aPos, aSize, aPresParam, lVisible )

   aSize := ::currentSize()
   ::fileList:create  (,, {4, 30}        , {aSize[1]-8, aSize[2]-30-4 } )
   ::fileList:itemMarked   := {|| ::fileMarked() }
   ::fileList:itemSelected := {|| ::fileSelected() }
   ::extList:create(,, {4, 30-4-24-96}, {aSize[1]-8, 24+96 } )

   ::aExtensions := { "*.*" }
   AEval( ::aExtensions, {|c| ::extList:xbpListBox:addItem( c ) } )

   ::extList:xbpListBox:setData( { 1 } )
   ::extList:xbpListBox:itemSelected := {|| ::fillFileList() }

RETURN self



/*
 * Verzeichnis und Dateierweiterungen fr die Dateiliste definieren
 */
METHOD FileSelector:setFilesTo( cDir, aExtensions )

   IF Valtype( aExtensions ) == "A"
      ::extList:xbpListBox:clear()
      AEval( aExtensions, {|c| ::extList:xbpListBox:addItem( c ) } )
      ::extList:xbpListBox:setData( { 1 } )
   ENDIF

   IF Valtype( cDir ) == "C"
      ::cDir := cDir
   ENDIF

   ::fillFileList()
RETURN self



/*
 * Listbox mit bereinstimmenden Dateien fllen
 */
METHOD FileSelector:fillFileList()
   LOCAL i, imax, j, jmax, aExtensions := {}
   LOCAL nOffset, cExtension


  /*
   * Liste der Erweiterungen aus der Combobox holen.
   * Als Trennzeichen dient das Leerzeichen
   */
   cExtension := AllTrim( ::extList:xbpSLE:getData() )

   DO WHILE ! Empty( cExtension )
      i := At( " ", cExtension )
      IF i > 0
         AAdd( aExtensions, AllTrim( SubStr( cExtension, 1, i-1 ) ) )
         cExtension := AllTrim( SubStr( cExtension, i+1 ) )
      ELSE
         AAdd( aExtensions, cExtension )
         cExtension := ""
      ENDIF
   ENDDO

   jmax := Len( aExtensions )

   ::fileList:clear()
   ::aFiles  := {}


  /*
   * Listbox mit Dateinamen fllen
   */
   FOR j:=1 TO jmax
      aDir    := Directory( ::cDir + "\" + aExtensions[j] )
      nOffset := Len( ::aFiles )
      imax    := Len( aDir )

      ASize( ::aFiles, nOffset + imax )

      FOR i:=1 TO imax
         ::aFiles[nOffset+i] := aDir[i,F_NAME]
         ::fileList:addItem( aDir[i,F_NAME] )
      NEXT
   NEXT

RETURN self



/*
 * Benutzerdefiniertes Ereignis "Datei ist markiert" plus Dateinamen senden
 */
METHOD FileSelector:fileMarked
   LOCAL aItems := ::fileList:getData()

   IF ! Empty( aItems ) .AND. ! Empty( ::aFiles )
      PostAppEvent( xbeFS_FileMarked, ::cDir + "\" + ::aFiles[ aItems[1] ] )
   ENDIF
RETURN self



/*
 * Benutzerdefiniertes Ereignis "Datei ist gewhlt" plus Dateinamen senden
 */
METHOD FileSelector:fileSelected
   LOCAL aItems := ::fileList:getData()

   IF ! Empty( aItems ) .AND. ! Empty( ::aFiles )
      PostAppEvent( xbeFS_FileSelected, ::cDir + "\" + ::aFiles[ aItems[1] ] )
   ENDIF

RETURN self



/*
 * Die Funktion gibt ein Array mit den Buchstaben aller verfgbarer
 * Laufwerken auer Diskettenlaufwerke zurck
 */
#define DRIVE_NOT_READY  21

FUNCTION AvailDrives()
   LOCAL cDrive := CurDrive()
   LOCAL aDrives:= { cDrive }
   LOCAL bError := ErrorBlock( {|oErr| DriveError( oErr, aDrives ) } )

   FOR i:=3 TO 26
      BEGIN SEQUENCE
        Curdrive( Chr(64+i) )
        CurDir()
        IF AScan( aDrives, CurDrive() ) == 0
           AAdd( aDrives, Curdrive() )
        ENDIF
      ENDSEQUENCE
   NEXT
   AEval(aDrives, {|x| x := UPPER(x) } )

   CurDrive( cDrive )

   ErrorBlock( bError )
RETURN ASort( aDrives )



/*
 * Laufzeitfehler abfangen, wenn ein Laufwerk nicht bereit ist
 */
STATIC PROCEDURE DriveError( oError, aDrives )
   IF oError:osCode == DRIVE_NOT_READY
      AAdd( aDrives, oError:args[1] )
   ENDIF
   BREAK
RETURN