**************************************************
*                                                *
*     PROYECTO: HTMLEditor                       *
*     ARCHIVOS: HtmlEditor.prg, HtmlEditor.ch,   *
*               HtmlEditor.arc                   *
*                                                *
*  Programador: J.A. Diego Kerejeta              *
*       Creado: 14-03-2011                       *
*                                                *
*      Version: 1.0.2 ; 12-06-2011               *
*                                                *
*   Changelog:                                   *
*               - Embedded manifest file.        *
*               - New button and context menu to *
*                 support hyperlinks.            *
*               - Custom dialog replaced by the  *
*                 Explorer's standard for image  *
*                 management.                    *
*               - Fixed double space when        *
*                 pressing Return.               *
*               - Fixed bug in setDefaultFont()  *
*                 method that caused a delay of  *
*                 three seconds to load the      *
*                 application.                   *
*                                                *
**************************************************

#include 'Appevent.ch'
#include 'Common.ch'
#include 'Xbp.ch'
#include 'activeX.ch'
#include 'gra.ch'
#include 'dll.ch'
#include 'htmleditor.ch'
#include 'ot4xb.ch'

#pragma library( "ot4xb.lib" )
#pragma library( "ascom10.lib"        )
#pragma library( "xppui2.lib"         )

#define PROTECTED_MODE

#define StatbarHeight   26
#define ToolbarHeight   32

**************
PROCEDURE Main
**************
LOCAL oForm

 SET ALTER TO LOGFILE.TXT

oForm:= HTMLEditor():new()
   oForm:minSize      := { 900, 200 }
   oForm:create()
   oForm:close        := {|u1, u2, obj| AppQuit( obj ) }
   oForm:setInputFocus:= {|u1, u2, obj| obj:setFocus() }
   oForm:show()

   CenterControl( oForm,AppDeskTop())

SetAppWindow( oForm )
SetAppFocus( oForm )

   CenterControl( oForm,AppDeskTop())


GenericHandleEvent( xbeP_Quit )

IF Valtype( GetApplication():mainForm ) = 'O' .AND. GetApplication():mainForm:status() = XBP_STAT_CREATE
   GetApplication():mainForm:destroy()
ENDIF

 SET ALTER TO
QUIT
RETURN

****************
PROCEDURE AppSys
****************
RETURN

************************
PROCEDURE AppQuit( obj )
************************
LOCAL nButton

IF obj:changed()
   nButton:= ConfirmBox( obj, TXT_CONFIR_CHANGES, TXT_TITLE_MAIN, XBPMB_YESNOCANCEL, XBPMB_QUESTION+XBPMB_APPMODAL+XBPMB_MOVEABLE )

   IF ( nButton = XBPMB_RET_YES .AND. obj:saveAs() ) .OR. nButton = XBPMB_RET_NO
      PostAppEvent( xbeP_Quit )
   ENDIF
ELSE // IF ConfirmBox( obj, TXT_CONFIR_QUIT, TXT_TITLE_MAIN, XBPMB_YESNO, XBPMB_QUESTION+XBPMB_APPMODAL+XBPMB_MOVEABLE ) = XBPMB_RET_YES
   PostAppEvent( xbeP_Quit )
ENDIF
RETURN

*******************************
CLASS HTMLEditor FROM XbpDialog
*******************************
PROTECTED:
   VAR _cOriginalData
   VAR oStatusbar
   VAR oToolbar
   VAR oFontSize
   VAR oFontName
   VAR cCurrentFile

   CLASS VAR nDocument SHARED

INLINE METHOD CheckSpell
   ************************
      LOCAL bHandler:= ::oIE:selectionChangeHandler
      LOCAL oCheck

      ::oIE:selectionChangeHandler:= nil

      oCheck:= XbpSpellChecker():new( nil, self )
         oCheck:oDoc:= ::oIE:document
         CenterControl( oCheck:create() )
         oCheck:showModal():destroy()

      ::oIE:selectionChangeHandler:= bHandler

   RETURN self

INLINE METHOD GetFileName( cTitle, aFilters, cDefExtension )
   ************************************************************
      LOCAL oDlg, cFile, cExt, nPos

      oDlg := XbpFileDialog():new( self )
         oDlg:title       := cTitle
         oDlg:fileFilters := aFilters
         oDlg:validatePath:= .T.
         IF cDefExtension <> nil
            oDlg:defExtension:= cDefExtension
         ENDIF
         oDlg:create()

      DO WHILE .T.
         cFile:= oDlg:open()
         IF cFile = nil
            EXIT
         ELSEIF ( nPos:= Rat( '.', cFile ) ) = 0
            ConfirmBox( self, TXT_NO_VALID_FILE, TXT_TITLE_MAIN, XBPMB_CANCEL, XBPMB_WARNING+XBPMB_APPMODAL+XBPMB_MOVEABLE )
         ELSE
            cExt:= Lower( Right( cFile, 1 + Len( cFile ) - nPos ) )
            nPos:= AScan( aFilters, {|aF| cExt $ Lower( aF[ 2 ] ) } )

            IF nPos = 0
               ConfirmBox( self, TXT_NO_VALID_FILE, TXT_TITLE_MAIN, XBPMB_CANCEL, XBPMB_WARNING+XBPMB_APPMODAL+XBPMB_MOVEABLE )
            ELSEIF ! FExists( cFile )
               ConfirmBox( self, TXT_ERR_NO_FILE, TXT_TITLE_MAIN, XBPMB_CANCEL, XBPMB_WARNING+XBPMB_APPMODAL+XBPMB_MOVEABLE )
            ELSE
               EXIT
            ENDIF
         ENDIF
      ENDDO
      oDlg:destroy()
   RETURN cFile

INLINE METHOD RefreshTitle
   **************************
      ::setTitle( IIF( ::cCurrentFile = nil, TXT_DEF_DOCU_NAME + Ltrim( Str( ++::nDocument ) ), ;
                                             ExtractFileName( ::cCurrentFile ) ) + ' - ' + TXT_TITLE_MAIN )
   RETURN self

EXPORTED:
   VAR oIE
   VAR cDefaultFontName
   VAR cDefaultFontSize

INLINE METHOD Destroy
   *********************
      ::oIE:document:designmode:= 'off'

      ::XbpDialog:destroy()
      ::_cOriginalData:= ::oStatusbar:= ::oToolbar:= ::oFontSize:= ::oFontName:= ;
      ::cCurrentFile:= ::oIE:= ::cDefaultFontName:= ::cDefaultFontSize:= nil
   RETURN

INLINE METHOD Init( oParent, oOwner, aPos, aSize, aPP, lVisible )
   *****************************************************************
      DEFAULT oParent  TO AppDesktop(), ;
              aPos     TO { 599, 253 }, ;
              aSize    TO { 700, 395 }, ;
              aPP      TO {}, ;
              lVisible TO .F.

      AAdd( aPP, { XBP_PP_COMPOUNDNAME, FONT_BASE          } )
      AAdd( aPP, { XBP_PP_ORIGIN,       XBP_ORIGIN_TOPLEFT } )

      ::XbpDialog:init( oParent, oOwner, aPos, aSize, aPP, lVisible )
         ::drawingArea:clipChildren:= .T.
         ::taskList                := .T.
         ::title                   := TXT_TITLE_MAIN

#IFDEF UseDXE
      ::oToolbar:= DXE_ToolBar():new( ::drawingArea,, { 0, 0 }, { aSize[ 1 ], ToolbarHeight } )
*   ::oToolbar:borderStyle  := XBPFRAME_NONE      // all other give WS_BORDER
*   ::oToolbar:textAlign    := XBPALIGN_BOTTOM    // or XBPALIGN_RIGHT
*   ::oToolbar:appearance   := XBP_APPEARANCE_3D  // or XBP_APPEARANCE_FLAT
*   ::oToolbar:wrappable    := .F.
    ::oToolbar:showToolTips := .T.
*   ::oToolbar:lNORESIZE    := .T.                // is CCS_NORESIZE

#ELSE
      ::oToolbar:= XbpToolBar():new( ::drawingArea,, { 0, 0 }, { aSize[ 1 ], ToolbarHeight } )
#ENDIF

//
// create in same order you want to use it
//

      ::oFontName:= XbpComboBox():new( ::oToolbar,, { 2, -150 }, { 232, 172 }, { { XBP_PP_BGCLR, XBPSYSCLR_ENTRYFIELD } } )
*     ::oFontName:= XbpComboBox():new()
         ::oFontName:type   := XBPCOMBO_DROPDOWNLIST
         ::oFontName:tabstop:= .T.

      ::oFontSize:= XbpComboBox():new( ::oToolbar,, { 1, -150 }, {  56, 172 }, { { XBP_PP_BGCLR, XBPSYSCLR_ENTRYFIELD } } )
*     ::oFontSize:= XbpComboBox():new()
         ::oFontSize:type   := XBPCOMBO_DROPDOWNLIST
         ::oFontSize:tabstop:= .T.


#IFDEF UseDXE

      ::oStatusbar:= DXE_StatBar():new( ::drawingArea )
      ::oStatusbar:DrawMode := XBP_DRAW_OWNER
      ::oStatusbar:drawItem := {| oPS, aInfo, oSelf | Colordraw(oPS, aInfo, oSelf) }

#ELSE
      ::oStatusbar:= XbpStatusBar():new( ::drawingArea )
#ENDIF

      ::oFontName:itemSelected:= {|u1, u2, obj| obj:cargo := Trim( obj:XbpSLE:getData() ), ::oIE:execCommand( CMD_FontName, .F., obj:cargo ) }
      ::oFontSize:itemSelected:= {|u1, u2, obj| obj:cargo := obj:getData()[ 1 ]          , ::oIE:execCommand( CMD_FontSize, .F., obj:cargo ) }
      ::close                 := {|| ::destroy() }
      ::cDefaultFontName      := 'Arial'
      ::cDefaultFontSize      := 'x-small'
      IF ::nDocument = nil;   ::nDocument:= 0;  ENDIF
   RETURN self

INLINE METHOD Create( oParent, oOwner, aPos, aSize, aPP, lVisible )
   *******************************************************************
      LOCAL aFonts, aFontNames, aFontSizes, oButton, oPanel, bError, oError, nCont, nLen, lExact
LOCAL hModule

      ::XbpDialog:create( oParent, oOwner, aPos, aSize, aPP, lVisible )

      aSize:= ::drawingArea:currentSize()

      ::oToolbar:create()
         ::oToolbar:allowCustomize:= .F.
         ::oToolbar:style         := XBPTOOLBAR_STYLE_FLAT
         ::oToolbar:borderStyle   := XBPFRAME_RECT
//
// create in same order you want to use it
//
      ::oFontName:create()
*     ::oFontName:create( ::oToolbar,, { 2, -150 }, { 232, 172 }, { { XBP_PP_BGCLR, XBPSYSCLR_ENTRYFIELD } } )
      ::oFontName:Cargo := "oFontName"
      ::oFontName:type := XBPCOMBO_SIMPLE

      ::oFontSize:create()
*     ::oFontSize:create( ::oToolbar,, { 1, -150 }, {  56, 172 }, { { XBP_PP_BGCLR, XBPSYSCLR_ENTRYFIELD } } )
      ::oFontSize:Cargo := "oFontSize"
      ::oFontSize:type := XBPCOMBO_SIMPLE

      ::oStatusbar:create(,, { 0, aSize[ 2 ] - StatbarHeight }, { aSize[ 1 ], StatbarHeight } )
         ::oStatusbar:caption:= TXT_PANEL

      ::oToolbar:loadImageSet( XBPTOOLBAR_STDIMAGES_SMALL )
      ::oToolbar:buttonClick:= {|oButton| ::ToolButtonClick( oButton ) }

      oButton               := ::oToolbar:addItem()
         oButton:image      := STD_IMAGE_FILEOPEN
         oButton:key        := KEY_OPEN
         oButton:tooltipText:= TIP_OPEN

      oButton               := ::oToolbar:addItem()
         oButton:image      := STD_IMAGE_FILESAVE
         oButton:key        := KEY_SAVE
         oButton:tooltipText:= TIP_SAVE

      oButton               := ::oToolbar:addItem()
         oButton:image      := STD_IMAGE_PRINT
         oButton:key        := KEY_PRINT
         oButton:tooltipText:= TIP_PRINT

      oButton               := ::oToolbar:addItem()
         oButton:image      := STD_IMAGE_PRINTPRE
         oButton:key        := KEY_PRINTPRE
         oButton:tooltipText:= TIP_PRINTPRE

      oButton               := ::oToolbar:addItem()
         oButton:style      := XBPTOOLBAR_BUTTON_SEPARATOR

      oButton               := ::oToolbar:addItem()
         oButton:style      := XBPTOOLBAR_BUTTON_PLACEHOLDER
         oButton:key        := KEY_FONTNAME
         nCont              := ::oToolBar:currentSize()[ 2 ] - ::oFontName:currentSize()[ 2 ] + ;
                               Int( abs( oButton:height - ::oFontName:sleSize()[ 2 ] ) / 2 )
*        ::oFontName:setPos( { oButton:left, nCont } )
         ::oFontName:setPos( {oButton:Left, oButton:Top - oButton:Height} )
         oButton:width      := ::oFontName:currentSize()[1]

      oButton               := ::oToolbar:addItem()
         oButton:style      := XBPTOOLBAR_BUTTON_PLACEHOLDER
         oButton:key        := KEY_FONTSIZE
*        ::oFontSize:setPos( { oButton:left, nCont } )
         ::oFontSize:setPos( {oButton:Left, oButton:Top - oButton:Height} )
         oButton:width      := ::oFontSize:currentSize()[1]

      oButton               := ::oToolbar:addItem()
         oButton:style      := XBPTOOLBAR_BUTTON_SEPARATOR

      oButton               := ::oToolbar:addItem()
         oButton:style      := XBPTOOLBAR_BUTTON_TOGGLE
         oButton:image      := BMP_BOLD
         oButton:key        := KEY_BOLD
         oButton:tooltipText:= TIP_BOLD

      oButton               := ::oToolbar:addItem()
         oButton:style      := XBPTOOLBAR_BUTTON_TOGGLE
         oButton:image      := BMP_ITALIC
         oButton:key        := KEY_ITALIC
         oButton:tooltipText:= TIP_ITALIC

      oButton               := ::oToolbar:addItem()
         oButton:style      := XBPTOOLBAR_BUTTON_TOGGLE
         oButton:image      := BMP_UNDERLINE
         oButton:key        := KEY_UNDERLINE
         oButton:tooltipText:= TIP_UNDERLINE

      oButton               := ::oToolbar:addItem()
         oButton:image      := BMP_FGCLR
         oButton:key        := KEY_FGCLR
         oButton:tooltipText:= TIP_FGCLR

      oButton               := ::oToolbar:addItem()
         oButton:image      := BMP_BGCLR
         oButton:key        := KEY_BGCLR
         oButton:tooltipText:= TIP_BGCLR

      oButton               := ::oToolbar:addItem()
         oButton:style      := XBPTOOLBAR_BUTTON_SEPARATOR

      oButton               := ::oToolbar:addItem()
         oButton:style      := XBPTOOLBAR_BUTTON_BUTTONGROUP
         oButton:image      := BMP_LEFT
         oButton:key        := KEY_LEFT
         oButton:tooltipText:= TIP_LEFT

      oButton               := ::oToolbar:addItem()
         oButton:style      := XBPTOOLBAR_BUTTON_BUTTONGROUP
         oButton:image      := BMP_CENTER
         oButton:key        := KEY_CENTER
         oButton:tooltipText:= TIP_CENTER

      oButton               := ::oToolbar:addItem()
         oButton:style      := XBPTOOLBAR_BUTTON_BUTTONGROUP
         oButton:image      := BMP_RIGHT
         oButton:key        := KEY_RIGHT
         oButton:tooltipText:= TIP_RIGHT

      oButton               := ::oToolbar:addItem()
         oButton:style      := XBPTOOLBAR_BUTTON_BUTTONGROUP
         oButton:image      := BMP_FULL
         oButton:key        := KEY_FULL
         oButton:tooltipText:= TIP_FULL

      oButton               := ::oToolbar:addItem()
         oButton:style      := XBPTOOLBAR_BUTTON_SEPARATOR

      oButton               := ::oToolbar:addItem()
         oButton:style      := XBPTOOLBAR_BUTTON_TOGGLE
         oButton:image      := BMP_BULLET
         oButton:key        := KEY_BULLET
         oButton:tooltipText:= TIP_BULLET

      oButton               := ::oToolbar:addItem()
         oButton:style      := XBPTOOLBAR_BUTTON_TOGGLE
         oButton:image      := BMP_NUMBEREDLIST
         oButton:key        := KEY_NUMBEREDLIST
         oButton:tooltipText:= TIP_NUMBEREDLIST

      oButton               := ::oToolbar:addItem()
         oButton:image      := BMP_LEFTINDENT
         oButton:key        := KEY_LEFTINDENT
         oButton:tooltipText:= TIP_LEFTINDENT

      oButton               := ::oToolbar:addItem()
         oButton:image      := BMP_RIGHTINDENT
         oButton:key        := KEY_RIGHTINDENT
         oButton:tooltipText:= TIP_RIGHTINDENT

      oButton               := ::oToolbar:addItem()
         oButton:style      := XBPTOOLBAR_BUTTON_SEPARATOR

      oButton               := ::oToolbar:addItem()
         oButton:image      := BMP_INSERTIMAGE
         oButton:key        := KEY_INSERTIMAGE
         oButton:tooltipText:= TIP_INSERTIMAGE

      oButton               := ::oToolbar:addItem()
         oButton:image      := BMP_INSERTLINK
         oButton:key        := KEY_INSERTLINK
         oButton:tooltipText:= TIP_INSERTLINK

      oButton               := ::oToolbar:addItem()
         oButton:image      := BMP_INSERTLINE
         oButton:key        := KEY_INSERTLINE
         oButton:tooltipText:= TIP_INSERTLINE

      oButton               := ::oToolbar:addItem()
         oButton:image      := BMP_CHECKSPELL
         oButton:key        := KEY_CHECKSPELL
         oButton:tooltipText:= TIP_CHECKSPELL

      oButton               := ::oToolbar:addItem()
         oButton:style      := XBPTOOLBAR_BUTTON_SEPARATOR

      oButton               := ::oToolbar:addItem()
         oButton:image      := STD_IMAGE_UNDO
         oButton:key        := KEY_UNDO
         oButton:tooltipText:= TIP_UNDO

      oButton               := ::oToolbar:addItem()
         oButton:image      := STD_IMAGE_REDO
         oButton:key        := KEY_REDO
         oButton:tooltipText:= TIP_REDO

      oPanel                := ::oStatusbar:getItem( 1 )
         oPanel:autoSize    := XBPSTATUSBAR_AUTOSIZE_SPRING

      oPanel                := ::oStatusbar:addItem()
         oPanel:style       := XBPSTATUSBAR_PANEL_CAPSLOCK
         oPanel:alignment   := XBPALIGN_HCENTER
         oPanel:autoSize    := XBPSTATUSBAR_AUTOSIZE_CONTENTS
         oPanel:width       -= 40

      oPanel                := ::oStatusbar:addItem()
         oPanel:style       := XBPSTATUSBAR_PANEL_NUMLOCK
         oPanel:alignment   := XBPALIGN_HCENTER
         oPanel:autoSize    := XBPSTATUSBAR_AUTOSIZE_CONTENTS
         oPanel:width       -= 40

      oPanel                := ::oStatusbar:addItem()
         oPanel:style       := XBPSTATUSBAR_PANEL_INSERT
         oPanel:alignment   := XBPALIGN_HCENTER
         oPanel:autoSize    := XBPSTATUSBAR_AUTOSIZE_CONTENTS
         oPanel:width       -= 40

      aFonts := XbpFont():new( ::lockPS() ):list()
         ::unlockPS()
         aFontNames:= {}
         AEval( aFonts, {|o| IF( AScan( aFontNames, o:familyName ) == 0, ;
                                 AAdd( aFontNames, o:familyName ), nil ) } )

         lExact:= Set( _SET_EXACT, .T. )
         aFontNames:= ASort( aFontNames )
         Set( _SET_EXACT, lExact )

         nLen:= Len( aFontNames )
         FOR nCont:= 1 TO nLen
            ::oFontName:addItem( aFontNames[ nCont ] )
         NEXT
         ::oFontName:Setdata(1)

      aFontSizes:= { '8', '10', '12', '14', '18', '24', '36' }

         nLen:= Len( aFontSizes )
         FOR nCont:= 1 TO nLen
            ::oFontSize:addItem( aFontSizes[ nCont ] )
         NEXT
         ::oFontSize:Setdata(1)

      aPos       := { 0, ::oToolBar:currentSize()[ 2 ] }
      aSize[ 2 ] -= aPos[ 2 ]
      aSize[ 2 ] -= StatbarHeight

#IFDEF UseDXE
      //
      // hm .. i have to use -::oToolBar:currentSize()[ 2 ] too ?
      //
*     aSize[ 2 ] -= ::oToolBar:currentSize()[ 2 ]
#ENDIF

#ifdef PROTECTED_MODE

      bError := ErrorBlock( {|e| Break( e ) } )
      BEGIN SEQUENCE

#endif
         ::oIE := MyHTMLViewer():new():create( ::drawingArea,, aPos, aSize )
             ::oIE:document:designmode:= 'on'
             ::oIE:setDefaultFont( ::cDefaultFontName, ::cDefaultFontSize )
             ::selectionChangeHandler()
             ::_cOriginalData            := ::oIE:getData()
             ::oIE:selectionChangeHandler:= {|| ::selectionChangeHandler() }

#ifdef PROTECTED_MODE

      RECOVER USING oError
         MsgBox( IIF( oError:subCode == 6500, TXT_ERR_CREATION1, TXT_ERR_CREATION2 + Ltrim( Str( oError:subCode ) ) + ')'  ), TXT_TITLE_MAIN )
         QUIT

      END SEQUENCE
      ErrorBlock( bError )

#endif

      ::refreshTitle()
      ::drawingArea:resize:= {|aOld, aNew | ::_resize( aOld, aNew ) }

   RETURN self

INLINE METHOD SaveAs
   ********************
      LOCAL oDlg, cFile, cExt, lMht

      oDlg := XbpFileDialog():new( self )
         oDlg:title       := TXT_TITLE_SAVE
         oDlg:validatePath:= .T.
         oDlg:defExtension:= 'mht'
         oDlg:fileFilters := { { TXT_MHT_FILES, '*.mht' }, ;
                               { TXT_HTM_FILES, '*.htm;*.html' }  }
         oDlg:create()

      DO WHILE .T.
         cFile:= oDlg:saveAs( ::cCurrentFile )

         IF cFile = nil
            EXIT
         ENDIF
         cExt:= Lower( Right( cFile, Len( cFile ) - Rat( '.', cFile ) ) )
         lMht:= ( cExt == 'mht' )

         IF lMht .OR. cExt == 'htm' .OR. cExt == 'html'
            IF ! cFile == ::cCurrentFile .AND. FExists( cFile ) .AND. ;
               ConfirmBox( self, TXT_CONF_OVERWRITE, TXT_TITLE_MAIN, XBPMB_OKCANCEL, XBPMB_WARNING+XBPMB_APPMODAL+XBPMB_MOVEABLE ) <> XBPMB_RET_OK
               EXIT
            ENDIF
            oDlg:destroy()
            ::oIE:document:title:= ExtractFileName( cFile )
            IIF( lMht, ::saveAsMHT( cFile ), FileWrite( cFile, ::oIE:document:body:outerHtml ) )
            ::cCurrentFile  := cFile
            ::_cOriginalData:= ::oIE:getData()
            ::refreshTitle()
            RETURN .T.
         ENDIF
         ConfirmBox( self, TXT_NO_VALID_FILE, TXT_TITLE_MAIN, XBPMB_CANCEL, XBPMB_INFORMATION+XBPMB_APPMODAL+XBPMB_MOVEABLE )
      ENDDO
      oDlg:destroy()
   RETURN .F.

INLINE METHOD SaveAsMHT( cFileName )
   ************************************
      LOCAL oMsg, oStreem, cFileTmp

      oMsg:= Createobject( 'CDO.Message' )

      IF oMsg = nil
         ConfirmBox( self, TXT_ERR_CREATION3, TXT_TITLE_MAIN, XBPMB_CANCEL, XBPMB_WARNING+XBPMB_APPMODAL+XBPMB_MOVEABLE )
      ELSE
         DO WHILE .T.
            cFileTmp:= Getenv( 'TEMP' ) + '\_' + StrZero( RandomInt( 9999999 ), 7, 0 ) + '.htm'
            IF ! FExists( cFileTmp )
               EXIT
            ENDIF
            Sleep( 5 )
         ENDDO
         FileWrite( cFileTmp, ::oIE:document:body:outerHtml )
         oMSG:createMHTMLBody( cFileTmp )
         FErase( cFileTmp )
         oStreem:= oMsg:getStream()
         oStreem:saveToFile( cFileName, adSaveCreateOverWrite )
         oStreem:destroy()
         oMsg:destroy()
      ENDIF
   RETURN self

INLINE METHOD ToolButtonClick( oButton )
   ****************************************
      DO CASE
         CASE oButton:key == KEY_OPEN          ;  ::open()
         CASE oButton:key == KEY_SAVE          ;  ::saveAs()
         CASE oButton:key == KEY_PRINT         ;  ::oIE:execWB( OLECMDID_PRINT,        OLECMDEXECOPT_PROMPTUSER, 0, 0 )
         CASE oButton:key == KEY_PRINTPRE      ;  ::oIE:execWB( OLECMDID_PRINTPREVIEW, OLECMDEXECOPT_PROMPTUSER, 0, 0 )
         CASE oButton:key == KEY_BOLD          ;  ::oIE:execCommand( CMD_Bold, .F., nil )
         CASE oButton:key == KEY_ITALIC        ;  ::oIE:execCommand( CMD_Italic, .F., nil )
         CASE oButton:key == KEY_UNDERLINE     ;  ::oIE:execCommand( CMD_Underline, .F., nil )
         CASE oButton:key == KEY_LEFT          ;  ::oIE:execCommand( CMD_JustifyLeft, .F., nil )
         CASE oButton:key == KEY_CENTER        ;  ::oIE:execCommand( CMD_JustifyCenter, .F., nil )
         CASE oButton:key == KEY_RIGHT         ;  ::oIE:execCommand( CMD_JustifyRight, .F., nil )
         CASE oButton:key == KEY_FULL          ;  ::oIE:execCommand( CMD_JustifyFull, .F., nil )
         CASE oButton:key == KEY_BULLET        ;  ::oIE:execCommand( CMD_InsertUnorderedList, .F., nil )
         CASE oButton:key == KEY_NUMBEREDLIST  ;  ::oIE:execCommand( CMD_InsertOrderedList, .F., nil )
         CASE oButton:key == KEY_LEFTINDENT    ;  ::oIE:execCommand( CMD_Outdent, .F., nil )
         CASE oButton:key == KEY_RIGHTINDENT   ;  ::oIE:execCommand( CMD_Indent, .F., nil )
         CASE oButton:key == KEY_FGCLR         ;  ::oIE:chooseFGcolor()
         CASE oButton:key == KEY_BGCLR         ;  ::oIE:chooseBGcolor()
         CASE oButton:key == KEY_INSERTIMAGE   ;  ::oIE:execCommand( CMD_InsertImage, .T. )
         CASE oButton:key == KEY_CHECKSPELL    ;  ::checkSpell()
         CASE oButton:key == KEY_UNDO          ;  ::oIE:execCommand( CMD_Undo, .F., nil )
         CASE oButton:key == KEY_REDO          ;  ::oIE:execCommand( CMD_Redo, .F., nil )
         CASE oButton:key == KEY_INSERTLINE    ;  ::oIE:execCommand( CMD_InsertHorizontalRule, .F., nil )
         CASE oButton:key == KEY_INSERTLINK    ;  ::oIE:execCommand( CMD_CreateLink, .T., nil )
      ENDCASE
   RETURN self

INLINE METHOD SelectionChangeHandler
   ************************************
      LOCAL oDoc, xData, oButton

#ifdef PROTECTED_MODE

      LOCAL bError:= ErrorBlock( {|e| Break( e ) } )

      BEGIN SEQUENCE

#endif
         oDoc:= ::oIE:document

         IF ! ( xData:= oDoc:queryCommandValue( CMD_FontName ) ) == ::oFontName:cargo
            ::oFontName:xbpSle:setData( IIF( xData <> nil, xData, '' ) )
            ::oFontName:cargo:= xData
         ENDIF
         IF ! ( xData:= oDoc:queryCommandValue( CMD_FontSize ) ) == ::oFontSize:cargo
            ::oFontSize:setData( IIF( xData <> nil, xData, 0 ) )
            ::oFontSize:cargo:= xData
         ENDIF
         IF ( xData:= oDoc:queryCommandValue( CMD_Bold ) ) <> ( oButton:= ::oToolbar:getItem( KEY_BOLD ) ):pressed
            oButton:pressed:= xData
         ENDIF
         IF ( xData:= oDoc:queryCommandValue( CMD_Italic ) ) <> ( oButton:= ::oToolbar:getItem( KEY_ITALIC ) ):pressed
            oButton:pressed:= xData
         ENDIF
         IF ( xData:= oDoc:queryCommandValue( CMD_Underline ) ) <> ( oButton:= ::oToolbar:getItem( KEY_UNDERLINE ) ):pressed
            oButton:pressed:= xData
         ENDIF
         IF oDoc:queryCommandValue( CMD_JustifyLeft       ) .AND. ! ::oToolbar:getItem( KEY_LEFT   ):pressed
            ::oToolbar:getItem( KEY_LEFT ):pressed:= .T.
         ELSEIF oDoc:queryCommandValue( CMD_JustifyCenter ) .AND. ! ::oToolbar:getItem( KEY_CENTER ):pressed
            ::oToolbar:getItem( KEY_CENTER ):pressed:= .T.
         ELSEIF oDoc:queryCommandValue( CMD_JustifyRight  ) .AND. ! ::oToolbar:getItem( KEY_RIGHT  ):pressed
            ::oToolbar:getItem( KEY_RIGHT ):pressed:= .T.
         ELSEIF oDoc:queryCommandValue( CMD_JustifyFull   ) .AND. ! ::oToolbar:getItem( KEY_FULL   ):pressed
            ::oToolbar:getItem( KEY_FULL ):pressed:= .T.
         ENDIF
         IF ( xData:= oDoc:queryCommandValue( CMD_InsertUnorderedList ) ) <> ( oButton:= ::oToolbar:getItem( KEY_BULLET ) ):pressed
            oButton:pressed:= xData
         ENDIF
         IF ( xData:= oDoc:queryCommandValue( CMD_InsertOrderedList ) ) <> ( oButton:= ::oToolbar:getItem( KEY_NUMBEREDLIST ) ):pressed
            oButton:pressed:= xData
         ENDIF

#ifdef PROTECTED_MODE

      END SEQUENCE
      ErrorBlock( bError )

#endif
   RETURN self

INLINE METHOD _Resize( aOldSize, aNewSize )
*******************************************
LOCAL aPos, oButton, nYPos
LOCAL aAbsPos  := ::CurrentPos()
LOCAL aAbsSize := ::CurrentSize()
LOCAL aDesk    := AppDeskTop():CurrentSize()

LOCAL iMax, i
LOCAL oBtn
LOCAL nWide := 0
LOCAL cTmp

   ::oToolbar:setSize( { aNewSize[ 1 ], ::oToolbar:currentSize()[ 2 ] } )
//
// my DXE_Toolbar might have different Size
//
// aPos   := { 0, ::oToolBar:currentPos()[ 2 ] }
   aPos   := { 0, ::oToolBar:currentSize()[ 2 ] }

   oButton:= ::oToolbar:getItem( KEY_FONTNAME )
   nYPos  := aPos[ 2 ] - ::oFontName:currentSize()[ 2 ] + Int( abs( oButton:height - ::oFontName:sleSize()[ 2 ] ) / 2 )

#IFDEF UseDXE
   //
   // DXE use ToolbarButton as Parent -> {0,y}
   // so nothing todo
   //

   //
   // hm .. i have to use -::oToolBar:currentPOS()[ 2 ] too ?
   //
   ::oIE:setPosAndSize( aPos, { aNewSize[ 1 ], aNewSize[ 2 ] - aPos[ 2 ] - StatbarHeight - ::oToolBar:currentPos()[ 2 ] } )

#ELSE
   //
   // original have Toolbar as Parent
   //
   ::oFontName:setPos( { oButton:left, nYPos } )
   ::oFontSize:setPos( { ::oToolbar:getItem( KEY_FONTSIZE ):left, nYPos } )

   ::oIE:setPosAndSize( aPos, { aNewSize[ 1 ], aNewSize[ 2 ] - aPos[ 2 ] - StatbarHeight } )
#ENDIF

   ::oStatusbar:setPosAndSize( { 0, aNewSize[ 2 ] - StatbarHeight }, { aNewSize[ 1 ], StatbarHeight } )
RETURN self

INLINE METHOD Open
   ******************
      LOCAL cFile, nButton, bHandler

      IF ::changed()
         nButton:= ConfirmBox( self, TXT_CONFIR_CHANGES, TXT_TITLE_MAIN, XBPMB_YESNOCANCEL, XBPMB_QUESTION+XBPMB_APPMODAL+XBPMB_MOVEABLE )

         IF ( nButton = XBPMB_RET_YES .AND. ! ::saveAs() ) .OR. nButton = XBPMB_RET_CANCEL
            RETURN self
         ENDIF
      ENDIF

      cFile:= ::getFileName( TXT_TITLE_OPEN, { { TXT_HTML_FILES  + ' (*.mht;*.htm;*.html)', '*.mht;*.htm;*.html' } }, 'mht' )

      IF cFile <> nil
         bHandler                    := ::oIE:selectionChangeHandler
         ::oIE:selectionChangeHandler:= nil
         ::oIE:document:designmode   := 'off'
         ::oIE:navigate( cFile )
         ::oIE:doWhileToNavigateComplete()
         ::oIE:document:designmode:= 'on'
         ::oIE:setDefaultFont( ::cDefaultFontName, ::cDefaultFontSize )
         ::selectionChangeHandler()
         ::oIE:selectionChangeHandler:= bHandler
         ::_cOriginalData            := ::oIE:getData()
         ::cCurrentFile              := cFile
         ::refreshTitle()
      ENDIF
   RETURN self

INLINE METHOD SetFocus
   **********************
#ifdef PROTECTED_MODE

      LOCAL bError:= ErrorBlock( {|e| Break( e ) } )

      BEGIN SEQUENCE

#endif

          ::oIE:setFocus()

#ifdef PROTECTED_MODE

      END SEQUENCE
      ErrorBlock( bError )

#endif
   RETURN self

INLINE METHOD Changed
   *********************
   RETURN ! ( ::_cOriginalData == ::oIE:getData() )
ENDCLASS

********************************************
STATIC CLASS MyHTMLViewer FROM XbpHTMLViewer
********************************************
PROTECTED:
   VAR oSubMenu
   VAR _interface
   VAR _document

   CLASS VAR lTerminalServer SHARED

INLINE METHOD GetColor( nColor, oDlg )
   **************************************
      LOCAL oColorStruct, paColors

      STATIC cCustColors:= nil

      DEFAULT nColor TO 0, ;
              oDlg   TO SetAppWindow()

      IF cCustColors = nil
         cCustColors:= Replicate( L2Bin( 16777215 ), 16 )
      ENDIF
      paColors:= _xGrab( cCustColors )

      IF paColors != 0
         oColorStruct:= COLORSTRUC():new()
            oColorStruct:lStructSize   := 36
            oColorStruct:hwnd          := oDlg:getHWND()
            oColorStruct:hInstance     := 0
            oColorStruct:rgbResult     := nColor
            oColorStruct:lpCustColors  := paColors
            oColorStruct:flags         := xCC_RGBINIT + xCC_FULLOPEN + IIF( 'Windows 9' $ Os(), xCC_SOLIDCOLOR, 0 )
            oColorStruct:lCustData     := 0
            oColorStruct:lpfnHook      := 0
            oColorStruct:lpTemplateName:= 0

         IF @COMDLG32:ChooseColorA( oColorStruct ) = 1
            nColor:= oColorStruct:rgbResult
         ENDIF
         cCustColors:= PeekStr( paColors, 0, 64 )
         _xfree( paColors )
      ENDIF
   RETURN nColor

EXPORTED:
   VAR selectionChangeHandler

INLINE METHOD Destroy
   *********************
      ::selectionChangeHandler:= nil

      ::XbpHTMLViewer:destroy()

      IF Valtype( ::_document ) = 'O'
          ::_document:destroy()
         ::_document:= ::_interface:= nil
      ENDIF
      ::oSubMenu:= nil
   RETURN self

INLINE METHOD Create( oParent, oowner, aPos, aSize, aPP, lVisible )
   *******************************************************************
      IF ::lTerminalServer = nil
         ::lTerminalServer:= ( GetSystemMetrics( SM_REMOTESESSION ) > 0 )
      ENDIF
      ::XbpHTMLViewer:create( oParent, oowner, aPos, aSize, aPP, lVisible )
      ::XbpHTMLViewer:navigate( 'about:<DIV></DIV>' ,,, .F. )
      ::doWhileToNavigateComplete()
   RETURN self

INLINE METHOD DocumentComplete( cUrl )
   **************************************
      IF ! ::busy .AND. Valtype( ::document ) = 'O' .AND. ::document:interface <> ::_interface
         IF Valtype( ::_document ) = 'O'
            ::_document:destroy()
            ::_document:= nil
         ENDIF
         ::_interface:= ::document:interface
         ::_document := ::document:dynamicCast( ActiveXObject() )
         ::_document:subscribeEvent( DISPID_ONCONTEXTMENU,     {|| ::contextMenuHandler() } )
         ::_document:subscribeEvent( DISPID_ONSELECTIONCHANGE, {|| IIF( ::selectionChangeHandler != nil,  EVAL( ::selectionChangeHandler ), nil ) } )
      ENDIF
   RETURN self

INLINE METHOD InheritPresParams   // Bug in XbpHTMLViewer
   *******************************
      LOCAL bError  := ErrorBlock( {|e| Break( e ) } )
      LOCAL lSuccess:= .F.

      BEGIN SEQUENCE
         lSuccess:= ::XbpHTMLViewer:inheritPresParams()
      ENDSEQUENCE

      ErrorBlock( bError )
   RETURN lSuccess

INLINE METHOD ContextMenuHandler
   ********************************
      LOCAL oPT, oRange, oParent, cType

      IF ::oSubMenu <> nil
         ::oSubMenu:destroy()
      ENDIF

      ::oSubMenu:= XbpMenu():new():create( self )
         ::oSubMenu:addItem( { MNU_CUT,       {|| ::cut()   } } )
         ::oSubMenu:addItem( { MNU_COPY,      {|| ::copy()  } } )
         ::oSubMenu:addItem( { MNU_PASTE,     {|| ::paste() } } )
         ::oSubMenu:addItem( { nil, nil, XBPMENUBAR_MIS_SEPARATOR } )
         ::oSubMenu:addItem( { MNU_SELECTALL, {|| ::execWB( OLECMDID_SELECTALL, OLECMDEXECOPT_DONTPROMPTUSER, 0, 0 ) } } )

      oPT   := ::document:selection
      cType := oPT:type
      oRange:= oPT:createRange()

      IF cType == 'Control'
         IF oRange:length > 0 .AND. oRange:item( 0 ):tagName == 'IMG'
            ::oSubMenu:addItem( { nil, nil, XBPMENUBAR_MIS_SEPARATOR } )
            ::oSubMenu:addItem( { MNU_IMG_PROPERTIES, {|| ::execCommand( CMD_InsertImage, .T. ) } } )
         ENDIF
      ELSE
         oParent:= oRange:parentElement()

         IF oParent:tagName == 'A'
            ::selectNode( oParent )
            ::oSubMenu:addItem( { nil, nil, XBPMENUBAR_MIS_SEPARATOR } )
            ::oSubMenu:addItem( { MNU_LNK_MODIFY, {|| ::execCommand( CMD_CreateLink, .T. ) } } )
            ::oSubMenu:addItem( { MNU_LNK_REMOVE, {|| ::execCommand( CMD_Unlink, .T. ) } } )
         ELSEIF cType = 'None'
            ::oSubMenu:disableItem( 1 )
            ::oSubMenu:disableItem( 2 )
         ENDIF
      ENDIF

      oPT:= ::document:parentWindow:event
         oPT:returnValue:= .F.

      ::oSubMenu:popup( self, { oPT:x, ::currentSize()[ 2 ] - oPT:y } )
   RETURN .T.

INLINE METHOD SelectNode( oNode )
   *********************************
      LOCAL bError, oRange

      bError := ErrorBlock( {|e| Break( e ) } )
      BEGIN SEQUENCE
         oRange:= ::document:body:createControlRange()
         oRange:addElement( oNode )
      RECOVER
         oRange:= ::document:body:createTextRange()
         oRange:moveToElementText( oNode )
      END SEQUENCE
      ErrorBlock( bError )
      oRange:select()
   RETURN self

INLINE METHOD DoWhileToNavigateComplete
   ***************************************
      LOCAL oXbp, mp1, mp2
      LOCAL nEvent:= 0
      LOCAL nCont := 0
      DO WHILE nEvent != xbeHTML_NavigateComplete
         nEvent:= AppEvent( @mp1, @mp2, @oXbp, 6000 )
         IF oXbp != nil;  oXbp:handleEvent( nEvent, mp1, mp2 )
         ELSE          ;  EXIT   ; ENDIF
      ENDDO

      DO WHILE ::_document = nil .AND. ++nCont < 30
         ::documentComplete()
         Sleep( 30 )
      ENDDO
   RETURN self

INLINE METHOD SetData( cTxt, cTitle )
   *************************************
      LOCAL nCont:= 0

      DO WHILE ++nCont < 30
         IF ::document:body != nil
            IF Empty( cTxt )
               cTxt:= '<DIV>&nbsp;</DIV>'
            ENDIF
            ::document:body:innerHtml:= cTxt
            cTxt                     := ::document:body:innerHtml
            EXIT
         ENDIF
         Sleep( 5 )
      ENDDO

      IF cTitle != nil
         ::document:title:= cTitle
      ENDIF
   RETURN cTxt

INLINE METHOD GetData
   *********************
      LOCAL cData:= ''
      LOCAL nCont:= 0

      DO WHILE ++nCont < 30
         IF ::document:body != nil
            cData:= ::document:body:innerHtml
            EXIT
         ENDIF
         Sleep( 5 )
      ENDDO
   RETURN cData

INLINE METHOD ChooseFGcolor
   ***************************
      LOCAL nOldColor:= ::document:queryCommandValue( CMD_ForeColor )
      LOCAL nNewColor:= ::getColor( nOldColor, GetParentForm( self ) )

      IF nOldColor != nNewColor
         ::execCommand( CMD_ForeColor, .F., nNewColor )
      ENDIF
      ::setFocus()
   RETURN self

INLINE METHOD ChooseBGcolor
   ***************************
      LOCAL nOldColor:= ::document:queryCommandValue( CMD_BackColor )
      LOCAL nNewColor:= ::getColor( nOldColor, GetParentForm( self ) )

      IF nOldColor != nNewColor
         ::execCommand( CMD_BackColor, .F., nNewColor )
      ENDIF
      ::setFocus()
   RETURN self

INLINE METHOD ExecCommand( cCommand, p1, p2 )
   *********************************************
#ifdef PROTECTED_MODE

      LOCAL bError := ErrorBlock( {|e| Break( e ) } )

      BEGIN SEQUENCE

#endif

         ::document:execCommand( cCommand, p1, p2 )

         IIF( ::selectionChangeHandler != nil,  EVAL( ::selectionChangeHandler ), nil )

#ifdef PROTECTED_MODE

      END SEQUENCE
      ErrorBlock( bError )

#endif

      ::setFocus()
   RETURN self

INLINE METHOD SetFocus
   **********************
      LOCAL nCont:= 0
      LOCAL oOld

      IF Valtype( ::document ) = 'O' .AND. ! ::document:hasFocus()
         oOld := SetAppFocus( self )
         ::document:focus()
         Sleep( 5 )
         DO WHILE ! ::document:hasFocus() .AND. nCont++ < 10
            ::document:focus()
            Sleep( 5 )
         ENDDO
      ENDIF
   RETURN IIF( oOld != nil, oOld, SetAppFocus() )

INLINE METHOD SetDefaultFont( cFont, cSize )
   ********************************************
      LOCAL nCont:= 0
      LOCAL oStyle

      DO WHILE ++nCont < 30
         IF ::document:body != nil .AND. ( oStyle:= ::document:body:style ) != nil
            oStyle:font       := 'normal ' + cSize + ' ' + cFont
            oStyle:borderWidth:= '1px'
            EXIT
         ENDIF
         Sleep( 5 )
      ENDDO
   RETURN self

INLINE METHOD Cut
   *****************
      IF ! ::lTerminalServer
         ::execWB( OLECMDID_CUT, OLECMDEXECOPT_DONTPROMPTUSER, 0, 0 ) // this does not work in Terminal Server 2003 (bug)
      ELSE
         ::setFocus()
         SendCtrlKey( VK_X )
      ENDIF
   RETURN self

INLINE METHOD Copy
   ******************
      IF ! ::lTerminalServer
         ::execWB( OLECMDID_COPY, OLECMDEXECOPT_DONTPROMPTUSER, 0, 0 ) // this does not work in Terminal Server 2003 (bug)
      ELSE
         ::setFocus()
         SendCtrlKey( VK_C )
      ENDIF
   RETURN self

INLINE METHOD Paste
   *******************
      IF ! ::lTerminalServer
         ::execWB( OLECMDID_PASTE, OLECMDEXECOPT_DONTPROMPTUSER, 0, 0 ) // this does not work in Terminal Server 2003 (bug)
      ELSE
         ::setFocus()
         SendCtrlKey( VK_V )
      ENDIF
   RETURN self
ENDCLASS

*****************************
PROCEDURE SendCtrlKey( xKey )
*****************************
LOCAL oInput := Input():New()
LOCAL nEvents:=  4
LOCAL nSize  := oInput:_sizeof_()
LOCAL pBuffer:= _xgrab( nSize * nEvents  )

oInput:_link_( pBuffer , .F.)
   oInput:type  := INPUT_KEYBOARD
   oInput:ki:wVk:= VK_CONTROL

GwstArrayNext(oInput)
   oInput:type  := INPUT_KEYBOARD
   oInput:ki:wVk:= xKey

GwstArrayNext(oInput)
   oInput:type      := INPUT_KEYBOARD
   oInput:ki:wVk    := VK_CONTROL
   oInput:ki:dwFlags:= KEYEVENTF_KEYUP

GwstArrayNext(oInput)
   oInput:type      := INPUT_KEYBOARD
   oInput:ki:wVk    := xKey
   oInput:ki:dwFlags:= KEYEVENTF_KEYUP

@user32:SendInput( nEvents, pBuffer, nSize )

oInput:_unlink_()
_xfree( pBuffer)
RETURN

**************************
BEGIN STRUCTURE COLORSTRUC
**************************
   MEMBER DWORD  lStructSize
   MEMBER HWND   hwnd
   MEMBER HWND   hInstance
   MEMBER DWORD  rgbResult
   MEMBER LPSTR  lpCustColors
   MEMBER DWORD  Flags
   MEMBER LPARAM lCustData
   MEMBER LONG   lpfnHook
   MEMBER LPSTR  lpTemplateName
END STRUCTURE

*********************
BEGIN STRUCTURE INPUT
*********************
   MEMBER DWORD type
   BEGIN UNION
   MEMBER @ MOUSEINPUT mi
   MEMBER @ KEYBDINPUT ki
   MEMBER @ HARDWAREINPUT hi
   END UNION
END STRUCTURE // 28 // BYTES

**************************
BEGIN STRUCTURE MOUSEINPUT
**************************
   MEMBER LONG dx
   MEMBER LONG dy
   MEMBER DWORD mouseData
   MEMBER DWORD dwFlags
   MEMBER DWORD time
   MEMBER DWORD dwExtraInfo
END STRUCTURE // 24 BYTES

**************************
BEGIN STRUCTURE KEYBDINPUT
**************************
   MEMBER WORD wVk
   MEMBER WORD wScan
   MEMBER DWORD dwFlags
   MEMBER DWORD time
   MEMBER DWORD dwExtraInfo
END STRUCTURE // 16 BYTES

*****************************
BEGIN STRUCTURE HARDWAREINPUT
*****************************
   MEMBER DWORD uMsg
   MEMBER WORD wParamL
   MEMBER WORD wParamH
END STRUCTURE // 8 BYTES




************************************
CLASS XbpSpellChecker FROM XbpDialog
************************************
PROTECTED
   VAR aSuggestions
   VAR oDocRange
   VAR cWordDelem
   VAR oWord
   VAR oPbSkip
   VAR oPbChange
   VAR oPbCancel
   VAR oLbSuggestions
   VAR Static1
   VAR Static2
   VAR Static3
   VAR Static4
   VAR oSleCurrentWrong
   VAR lAddSpace
   VAR nSubjectOffset

INLINE METHOD FillListBox
   *************************
      LOCAL nLen   := ::oLbSuggestions:numItems()
      LOCAL bMarked:= ::oLbSuggestions:itemMarked
      LOCAL nCont

      ::oLbSuggestions:itemMarked:= nil

      FOR nCont:= nLen TO 1 STEP -1
         ::oLbSuggestions:delItem( nCont )
      NEXT

      IF ! Empty( ::aSuggestions )
         nLen:= Len( ::aSuggestions )

         FOR nCont:= 1 TO nLen
            ::oLbSuggestions:addItem( ::aSuggestions[ nCont ] )
         NEXT

         ::oLbSuggestions:setData( 1 )
         ::oSleCurrentOk:setData( ::aSuggestions[ 1 ] )
      ELSE
         ::oSleCurrentOk:setData( '' )
      ENDIF
      ::oLbSuggestions:itemMarked:= bMarked
   RETURN self

INLINE METHOD MarkCurrentWrong
   ******************************
      ::oDocRange:select()
   RETURN self

INLINE METHOD NextWord
   **********************
      LOCAL cWord

      DO WHILE ::oDocRange:expand( 'word' )
         cWord:= ::oDocRange:text
         IF ! ::isDelemeter( Alltrim( cWord ) )
            ::lAddSpace:= ( Right( cWord, 1 ) == ' ' )
            RETURN cWord
         ENDIF
         ::oDocRange:collapse()
         ::oDocRange:moveStart( 'word', 1 )
      ENDDO
   RETURN nil

INLINE METHOD IsDelemeter( cWord )
   **********************************
   RETURN cWord $ ::cWordDelem

EXPORTED:
   VAR oSleCurrentOk
   VAR oDoc

INLINE METHOD Destroy
   *********************
      ::XbpDialog:destroy()

      ::aSuggestions:= ::oDocRange:= ::cWordDelem:= ::oWord:= ::oPbSkip:= ::oPbChange:= ;
      ::oPbCancel:= ::oLbSuggestions:= ::Static1:= ::Static2:= ::Static3:= ::Static4:= ;
      ::oSleCurrentWrong:= ::lAddSpace:= ::nSubjectOffset:= ::oSleCurrentOk:= ::oDoc:= nil
   RETURN self

INLINE METHOD Init( oParent, oOwner, aPos, aSize, aPP, lVisible )
   *****************************************************************
      DEFAULT oParent  TO AppDesktop(), ;
              lVisible TO .F., ;
              aPP      TO { { XBP_PP_COMPOUNDNAME, FONT_BASE } }

      ::XbpDialog:init( oParent, oOwner, aPos, aSize, aPP, lVisible )
         ::title              := TXT_TITLE_SPELL
         ::clientSize         := { 470, 270 }
         ::taskList           := .F.
         ::maxButton          := .F.
         ::border             := XBPDLG_DLGBORDER

      ::Static1:= XbpStatic():new( ::drawingArea, , { 12, 192 }, { 104, 36 }, { { XBP_PP_BGCLR, -255 } } )
         ::Static1:caption     := TXT_UNKNOWN
         ::Static1:clipSiblings:= .T.
         ::Static1:options     := XBPSTATIC_TEXT_WORDBREAK

      ::Static2:= XbpStatic():new( ::drawingArea, , { 12, 168 }, { 104, 24 }, { { XBP_PP_BGCLR, -255 } } )
         ::Static2:caption     := TXT_CHANGE
         ::Static2:clipSiblings:= .T.
         ::Static2:options     := XBPSTATIC_TEXT_VCENTER

      ::Static3:= XbpStatic():new( ::drawingArea, , { 12, 144 }, { 104, 24 }, { { XBP_PP_BGCLR, -255 } } )
         ::Static3:caption     := TXT_BUTTON_SUGGES
         ::Static3:clipSiblings:= .T.
         ::Static3:options     := XBPSTATIC_TEXT_VCENTER


      ::Static4:= XbpStatic():new( ::drawingArea, , { 0, 0 }, { 480, 24 }, { { XBP_PP_BGCLR, GRA_CLR_PALEGRAY } } )
         ::Static4:clipSiblings:= .T.
         ::Static4:options     := XBPSTATIC_TEXT_VCENTER

      ::oSleCurrentWrong:= XbpSLE():new( ::drawingArea, , { 116, 204 }, { 335, 24 }, { { XBP_PP_DISABLED_FGCLR, -58 } } )

      ::oSleCurrentOk:= XbpSLE():new( ::drawingArea, , { 116, 168 }, { 184, 24 }, { { XBP_PP_BGCLR, XBPSYSCLR_ENTRYFIELD }, ;
                                       { XBP_PP_DISABLED_FGCLR, -58 }, { XBP_PP_DISABLED_BGCLR, -255 } } )
         ::oSleCurrentOk:tabStop := .T.

      ::oLbSuggestions:= XbpListBox():new( ::drawingArea, , { 116, 36 }, { 184, 132 }, { { XBP_PP_BGCLR, XBPSYSCLR_ENTRYFIELD }, ;
                                           { XBP_PP_DISABLED_FGCLR, -58 }, { XBP_PP_DISABLED_BGCLR, -255 } } )
         ::oLbSuggestions:tabStop := .T.
         ::oLbSuggestions:itemMarked:= {|a,e,obj| ::oSleCurrentOk:setData( obj:getItem( obj:getData()[ 1 ] ) ) }

      ::oPbSkip  := XbpPushbutton():new( ::drawingArea, , { 312, 168 }, { 140, 24 }, { { XBP_PP_BGCLR, XBPSYSCLR_BUTTONMIDDLE }, { XBP_PP_FGCLR, -58 } } )
         ::oPbSkip:caption := TXT_BUTTON_IGNORE
         ::oPbSkip:activate:= {|| ::nextSuggestion( NEXT_WORD ) }

      ::oPbChange:= XbpPushbutton():new( ::drawingArea, , { 312, 132 }, { 140, 24 }, { { XBP_PP_BGCLR, XBPSYSCLR_BUTTONMIDDLE }, { XBP_PP_FGCLR, -58 } } )
         ::oPbChange:caption := TXT_BUTTON_REPLACE
         ::oPbChange:activate:= {|| ::replaceMarked() }

      ::oPbCancel:= XbpPushbutton():new( ::drawingArea, , { 312,  96 }, { 140, 24 }, { { XBP_PP_BGCLR, XBPSYSCLR_BUTTONMIDDLE }, { XBP_PP_FGCLR, -58 } } )
         ::oPbCancel:caption := TXT_BUTTON_CANCEL
         ::oPbCancel:activate:= {|| PostAppEvent( xbeP_Close,,, self ) }

      ::cWordDelem    :=  CR + LF + TAB + SPACE + COMMA + SEMICOLON + FULLSTOP + DOUBLEPOINT
      ::nSubjectOffset:= 1
   RETURN self

INLINE METHOD Create( oParent, oOwner, aPos, aSize, aPP, lVisible )
   *******************************************************************
      ::XbpDialog:create( oParent, oOwner, aPos, aSize, aPP, lVisible )

      ::Static1:create()
      ::Static2:create()
      ::Static3:create()
      ::Static4:create()

      ::oSleCurrentWrong:create():disable()
      ::oSleCurrentOk:create():disable()
      ::oLbSuggestions:create():disable()

      ::oPbSkip:create():disable()
      ::oPbChange:create():disable()
      ::oPbCancel:create():disable()
   RETURN self

INLINE METHOD NextSuggestion( lGoToNextWord )
   *********************************************
      LOCAL cWord, oSuggestions, nCont, nCount
      LOCAL lExit:= .F.

      ::oPbSkip:disable()
      ::oPbChange:disable()
      ::oPbCancel:disable()
      ::oSleCurrentOk:disable()
      ::oLbSuggestions:disable()

      ::static4:setCaption( TXT_CHECKING )

      ::oSleCurrentWrong:setData( '' )
      ::oDocRange:collapse()
      ::oDocRange:execCommand( 'Unselect' )

      IF lGoToNextWord
         ::oDocRange:moveStart( 'word', 1 )
      ENDIF

      DO WHILE ! lExit
         cWord:= ::nextWord()

         IF cWord = nil
            ::static4:setCaption( TXT_PADDING + TXT_END2_SPELL )
            ConfirmBox( self, TXT_END2_SPELL, TXT_TITLE_MAIN, XBPMB_OK, XBPMB_INFORMATION+XBPMB_APPMODAL+XBPMB_MOVEABLE )
            PostAppEvent( xbeP_Close,,, self )
            lExit:= .T.
         ELSEIF ! ::oWord:checkSpelling( cWord )
            oSuggestions  := ::oWord:getSpellingSuggestions( cWord )
            nCount        := oSuggestions:count
            ::aSuggestions:= {}

            FOR nCont:= 1 TO nCount
                AAdd( ::aSuggestions, oSuggestions:item( nCont ):name )
            NEXT

            ::oSleCurrentWrong:setData( cWord )
            ::fillListBox()
            ::markCurrentWrong()
            lExit:= .T.

            ::oPbSkip:enable()
            ::oPbChange:enable()
            ::oPbCancel:enable()
            ::oSleCurrentOk:enable()
            ::oLbSuggestions:enable()

            ::static4:setCaption( '' )

            SetAppFocus( ::oSleCurrentOk )
         ELSE
            ::oDocRange:collapse()
            ::oDocRange:moveStart( 'word', 1 )
         ENDIF
      ENDDO
   RETURN self

INLINE METHOD ReplaceMarked
   ***************************
      LOCAL cWord:= Alltrim( ::oSleCurrentOk:getData() )

      IF ! cWord == ''
         ::oDocRange:text:= cWord + IIF( ::lAddSpace, ' ', '' )
         ::oDocRange:moveStart( 'word', -1 )
         ::nextSuggestion( SAME_WORD )
      ENDIF
   RETURN self

INLINE METHOD ShowModal
   ***********************
      LOCAL cTxt

      ::setModalState( XBP_DISP_APPMODAL )
      ::show()
      SetAppFocus( ::oSleCurrentOk )

      ::oDocRange:= ::oDoc:body:createTextRange()
      cTxt       := Alltrim( ::oDocRange:text )

      IF Empty( cTxt ) .OR. ::isDelemeter( cTxt )
          ConfirmBox( self, TXT_END1_SPELL, TXT_TITLE_MAIN, XBPMB_OK, XBPMB_INFORMATION+XBPMB_APPMODAL+XBPMB_MOVEABLE )
      ELSE
         ::static4:setCaption( TXT_LOADING )

         ::oWord := CreateObject( 'Word.Application' )

         IF ::oWord = nil
            ConfirmBox( self, TXT_ERR_CREATION4, TXT_TITLE_MAIN, XBPMB_CANCEL, XBPMB_CRITICAL+XBPMB_APPMODAL+XBPMB_MOVEABLE )
         ELSE

            ::oWord:documents:add()
            ::oWord:displayAlerts:= wdAlertsNone

            ::nextSuggestion( FIRST_WORD )

            GenericHandleEvent( xbeP_Close, 60 )

            ::oWord:quit( wdDoNotSaveChanges )
            Sleep( 20 )
            ::oWord:destroy()
         ENDIF
      ENDIF
      ::setModalState( XBP_DISP_MODELESS )

      ::oDocRange:execCommand( 'Unselect' )
      ::oDocRange:destroy()
   RETURN self
ENDCLASS

**********************************************************
PROCEDURE GenericHandleEvent( nEndEvent, nInactivitySecs )
**********************************************************
LOCAL nEvent, mp1, mp2, oXbp, nSecs

IF nInactivitySecs = nil
   DO WHILE nEvent <> nEndEvent
      nEvent := AppEvent( @mp1, @mp2, @oXbp )
      oXbp:HandleEvent( nEvent, mp1, mp2 )
   ENDDO
ELSE
   nSecs:= 0

   DO WHILE nEvent <> nEndEvent
      nEvent := AppEvent( @mp1, @mp2, @oXbp, 100 )

      IF oXbp = nil
         IF ++nSecs > nInactivitySecs
            EXIT
         ENDIF
      ELSE
         nSecs:= 0

         oXbp:HandleEvent( nEvent, mp1, mp2 )
      ENDIF
   ENDDO
ENDIF
RETURN

*************************************
FUNCTION ExtractFileName( cFullName )
*************************************
LOCAL nPos := RAt( '\', cFullName )
LOCAL cName:= IIF( nPos > 0 .OR. ( nPos:= Rat( '/', cFullName ) ) > 0, SubStr( cFullName, nPos + 1 ), cFullName )

nPos:= RAt( '.', cName )

RETURN IIF( nPos = 0, cName, Left( cName, nPos - 1 ) )

***************************************************************************
STATIC DLLFUNCTION GetSystemMetrics( nIndex ) USING STDCALL FROM USER32.DLL
***************************************************************************


FUNCTION Colordraw(oPS, aInfo, oSelf)
LOCAL nItem    := aInfo[ XBP_DRAWINFO_ITEM   ] // Index des Elements, das neu gezeichnet werden muss
LOCAL nAction  := aInfo[ XBP_DRAWINFO_ACTION ] // Aktion, die die Nachricht xbeP_DrawItem auslste
LOCAL nState   := aInfo[ XBP_DRAWINFO_STATE  ] // Aktueller Anzeigestatus des Elements
LOCAL aRect    := aInfo[ XBP_DRAWINFO_RECT   ] // Ausgaberechteck fr die Zeichenoperation

LOCAL cText
LOCAL aPrev
LOCAL aSAttrs  := Array( GRA_AS_COUNT )
LOCAL aAAttrs  := Array( GRA_AA_COUNT )
LOCAL nXDest   := aInfo[XBP_DRAWINFO_RECT][1]
LOCAL nYDest   := aInfo[XBP_DRAWINFO_RECT][2]
LOCAL nWidth   := aInfo[XBP_DRAWINFO_RECT][3]
LOCAL nHeight  := aInfo[XBP_DRAWINFO_RECT][4]

   cText := aInfo[5]                             // extra element

   //
   // this calculation is different to Xbase++ !!!
   //
   // it is while i "simulate" oPS with ViewPoint 0,0
   // to use GRA Function with Bitmap Transfer to hDC
   //
   nWidth   := nWidth  - nXDest
   nHeight  := nHeight - nYDest
   nXDest   := 0
   nYDest   := 0

   aAAttrs[GRA_AA_COLOR] := XBPSYSCLR_3DFACE
   GraSetAttrArea( oPS, aAAttrs )
   GraBox( oPS, {nXDest,nYDest}, {nWidth,nHeight}, GRA_FILL )

   DO CASE
      CASE cText = "INS ON"     ; aSAttrs[GRA_AS_COLOR] := GRA_CLR_BLACK
      CASE cText = "NUM ON"     ; aSAttrs[GRA_AS_COLOR] := GRA_CLR_BLACK
      CASE cText = "CAPS ON"    ; aSAttrs[GRA_AS_COLOR] := GRA_CLR_BLACK
      CASE cText = "SCROLL ON"  ; aSAttrs[GRA_AS_COLOR] := GRA_CLR_BLACK

      CASE cText = "INS OFF"    ; aSAttrs[GRA_AS_COLOR] := GRA_CLR_DARKGRAY
      CASE cText = "NUM OFF"    ; aSAttrs[GRA_AS_COLOR] := GRA_CLR_DARKGRAY
      CASE cText = "CAPS OFF"   ; aSAttrs[GRA_AS_COLOR] := GRA_CLR_DARKGRAY
      CASE cText = "SCROLL OFF" ; aSAttrs[GRA_AS_COLOR] := GRA_CLR_DARKGRAY
   OTHERWISE
      aSAttrs[GRA_AS_COLOR] := XBPSYSCLR_WINDOWTEXT
   ENDCASE

   aSAttrs[ GRA_AS_HORIZALIGN ]  := GRA_HALIGN_RIGHT
   aSAttrs[ GRA_AS_VERTALIGN ]   := GRA_VALIGN_HALF
   aSAttrs[ GRA_AS_ANGLE ] := { 1, 0 }
   aPrev := GraSetAttrString( oPS, aSAttrs )

   GraCaptionStr(oPS, {nXDest,nYDest } ,;
                      {nWidth,nHeight} ,;
                       cText           ,;
                       XBPALIGN_VCENTER  )

   GraSetAttrString( oPS, aPrev )

RETURN .T.
